import Vue from 'vue'
import Vuex from 'vuex'
import api from '../services/api'

Vue.use(Vuex)

const shipment_skeleton = {
  weight: 1000,
  length: 150,
  width:  100,
  height: 50,
  product_id: null,
  signature_on_delivery: false,
  extra_cover: 0,
  price: {
    calculated_gst: 0,
    calculated_price: 0,
    calculated_price_ex_gst: 0,
    product_type: '',
  },
  tracking_details: {
    article_id: '',
    consignment_id: '',
  },
  label_details: {
    shipment_id: '',
    item_id: '',
  },
  tracking_events: [{
    date: '',
    description: '',
    location: '',
  }],
  tracking_status: '',
  clearance_items: [
    {
      description: '',
      quantity: 1,
      value: 1,
      country_of_origin: 'AU',
    }
  ]
}

export default new Vuex.Store({
  state: {
    postage_products: window.postage_products,
    order_id: window.order_id,
    errors: [],
    success_messages: [],
    from_address: {
      title: 'From',
      name: '',
      street: '',
      city: '',
      state: '',
      postcode: '',
      country: '',
    },
    to_address: {
      title: 'To',
      name: '',
      street: '',
      city: '',
      state: '',
      postcode: '',
      country: '',
    },
    shipments: [],
    shipments_ordered: false,
    has_dangerous_goods: false,
    label_request_id: '',
    label_url: '',
    order_subtotal: 0
  },
  mutations: {
    add_success_message (state, message) {
      state.success_messages.push(message)
    },
    clear_success_messages(state) {
      state.success_messages = []
    },
    add_shipment (state, shipment) {
      state.shipments.push(shipment)
    },
    add_tracking_details_to_shipments (state, shipments) {
      shipments.map((shipment, index) => {
        state.shipments[index].tracking_details.article_id = shipment.article_id
        state.shipments[index].tracking_details.consignment_id = shipment.consignment_id
        state.shipments[index].label_details.shipment_id = shipment.shipment_id
        state.shipments[index].label_details.item_id = shipment.item_id
      });
    },
    new_shipment(state) {
      state.shipments.push({ ...shipment_skeleton });
    },
    new_shipment_clearance_item (state, shipment_index) {
      const item = JSON.parse(JSON.stringify(shipment_skeleton.clearance_items[0]));
      item.value = state.order_subtotal
      item.description = state.order_description
      state.shipments[shipment_index].clearance_items.push(item)
    },
    delete_shipment_clearance_item (state, shipment_index, item_index) {
      state.shipments[shipment_index].clearance_items.splice(item_index, 1);
    },
    delete_shipment (state, index) {
      if (state.shipments.length > 1) {
        state.shipments.splice(index - 1, 1)
      }
    },
    add_prices_to_shipments (state, prices) {
      prices.forEach(function (price, index) {
        state.shipments[index].price = price
      });
    },
    add_label_details_to_shipments (state, labels) {
      labels.map((label) => {
        // it is always one.
        state.label_request_id = label.request_id
      });
    },
    add_label_url (state, url) {
        state.label_url = url
    },
    load_app_data (state, {packages,shipments_ordered, defaults, order_subtotal, order_description, from_address, label_request_id, label_url, shipments, to_address, has_dangerous_goods}) {

      state.order_subtotal = order_subtotal
      shipment_skeleton.clearance_items[0].value = order_subtotal
      shipment_skeleton.clearance_items[0].description = order_description
      if(packages.length > 0 && shipments.length === 0) {
        packages.map(function(shipment_package) {
          let shipment = shipment_skeleton
          shipment.weight = shipment_package.weight
          shipment.length = shipment_package.length
          shipment.width = shipment_package.width
          shipment.height = shipment_package.height
          state.shipments.push(shipment)
        })
      }

      if (defaults) {
        shipment_skeleton.length = defaults.dimensions.length;
        shipment_skeleton.width  = defaults.dimensions.width;
        shipment_skeleton.height = defaults.dimensions.height;
        shipment_skeleton.weight = defaults.weight;
      }

      if (shipments.length === 0 && packages.length === 0) {
        state.shipments.push(shipment_skeleton)
      }

      state.shipments_ordered = shipments_ordered
      state.has_dangerous_goods = has_dangerous_goods
      if (shipments.length > 0) {
        state.shipments = shipments
      }

      if (label_request_id) {
        state.label_request_id = label_request_id
      }

      if (label_url) {
        state.label_url = label_url
      }

      if (from_address) {
          state.from_address.name = from_address.name
          state.from_address.street = from_address.lines.join(' ')
          state.from_address.postcode = from_address.postcode
          state.from_address.city = from_address.suburb
          state.from_address.state = from_address.state
          state.from_address.country = from_address.country
      }

      if (to_address) {
        state.to_address.name = to_address.name
        state.to_address.street = to_address.lines.join(' ')
        state.to_address.postcode = to_address.postcode
        state.to_address.city = to_address.suburb
        state.to_address.state = to_address.state
        state.to_address.country = to_address.country
      }
    },
    add_errors (state, errors) {
      state.errors = errors
    }
  },
  actions: {
    get_prices () {
      return api.post( '', {
        action: "calculate_prices",
        order_id: this.state.order_id,
        shipments: this.state.shipments,
        has_dangerous_goods: this.state.has_dangerous_goods,
      }).then((response) => {
        if (response.data) {
          this.commit('add_prices_to_shipments', response.data)
        }
      }).catch(() => {});
    },
    create_shipments () {
      return api.post( '', {
        action: "create_shipments",
        order_id: this.state.order_id,
        shipments: this.state.shipments,
        has_dangerous_goods: this.state.has_dangerous_goods,
      }).then((response) => {
        if (response.data) {
          this.commit('add_tracking_details_to_shipments', response.data)
        }

        if(response.data.errors) {
          throw new Error();
        }
      });
    },
    generate_labels () {
      return api.post( '', {
        action: "generate_labels",
        order_id: this.state.order_id,
      }).then((response) => {
        if (response.data.labels) {
          this.commit('add_label_details_to_shipments', response.data.labels)
          this.commit('add_label_url', response.data.labels[0].url);
        }
      });
    },
    reset_prices () {
      const new_price = {
        calculated_gst: 0,
        calculated_price: 0,
        calculated_price_ex_gst: 0,
        product_type: '',
      }
      this.state.shipments.map ((shipment) => shipment.price = new_price)
    },
    load_app_data () {
      return api.post( '', {
        action: "load_app_data",
        order_id: this.state.order_id,
      }).then((response) => {
        this.commit('load_app_data', response.data)
      });
    }
  },
  getters: {
    from_address: state => {
      return state.from_address
    },
    to_address: state => {
      return state.to_address
    },
    shipments: state => {
      return state.shipments
    },
    postage_products: state => {
      return state.postage_products
    },
    label_request_id: state => {
      return state.label_request_id
    },
    label_url: state => {
      return state.label_url
    },
    order_subtotal: state => {
      return state.order_subtotal
    },
    shipments_ordered: state => {
      return state.shipments_ordered
    },
    errors: state => {
      return state.errors
    },
    success_messages: state => {
      return state.success_messages
    },
  }
})
